#region using
using System;
using System.Collections;
using System.Data.OleDb;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;

using GBPVR.Public;
using GBPVR.Backend.Common;
#endregion

namespace ExternalRecorder
{
	/// <summary>
	/// Summary description for ExternalRecorder.
	/// </summary>
	public class ExternalRecorder : IRecordingSource, IRecordingSourceConfiguration
	{

		#region Constructor / Destructor

		public ExternalRecorder()
		{
			// constructor
			string version = Assembly.GetExecutingAssembly().GetName().Version.ToString();
			Logger.Verbose("ExternalRecorder[" + this.GetHashCode() + "]: ExternalRecorder(version=" + version + ")");
			config = new ExternalRecorderConfig(this);
		}

		public ExternalRecorder(ExternalRecorderConfig config)
		{
			// constructor
			string version = Assembly.GetExecutingAssembly().GetName().Version.ToString();
			Logger.Verbose("ExternalRecorder[" + this.GetHashCode() + "]: ExternalRecorder(version=" + version + ", config=[" + config.GetHashCode() + "])");
			this.config = config;
		}

		#endregion

		#region Private Variables

		private string filename;

		internal ExternalRecorderConfig config;

		/// <summary> current status of recorder. </summary>
		private	RecordingStatus			currentStatus = RecordingStatus.RECORDING_FAILED;

		/// <summary> process launched to do the recording. </summary>
		private Process process;

		#endregion

		#region Private Methods

		private void startProgram(string parameters, string filename, ChannelTuning channelTuning, int minutes, bool previewMode)
		{
			this.filename = filename;

			string channelnumber;
			if (config.zeroPad)
			{
				if (config.threeDigit)
				{
					channelnumber = channelTuning.analogChannelNumber.ToString("000");
				} 
				else 
				{
					channelnumber = channelTuning.analogChannelNumber.ToString("00");
				}
			}
			else
			{
				channelnumber = channelTuning.analogChannelNumber.ToString();
			}
			if (parameters.IndexOf("{channel}") == -1)
			{
				parameters = parameters + " " + channelnumber;
			}
			else
			{
				parameters = parameters.Replace("{channel}", channelnumber);
			}
			/*if (parameters.IndexOf("{channelname}") != -1)
			{
				string channelname = string.Empty;
				
				ScheduleHelper scheduleHelper = ScheduleHelper.getInstance();
				IList channels = scheduleHelper.GetListingsForTimePeriod(DateTime.Now, DateTime.Now);
				foreach (Channel channel in channels) 
				{
					scheduleHelper.GetChannelByOID(channel.getOID);

					Channel.ChannelSource source = channel.getChannelSourceSpecifics(config.oid);
					if ((source != null) && (source.channelNumber == channelTuning.analogChannelNumber))
						channelname = channel.getName();
				}
				
				parameters = parameters.Replace("{channelname}", channelname);
			}*/
			if (parameters.IndexOf("{filename}") == -1)
			{
				//parameters = parameters + " " + filename;
			}
			else
			{
				parameters = parameters.Replace("{filename}", filename);
			}
			if (parameters.IndexOf("{minutes}") == -1)
			{
				//parameters = parameters + " " + minutes.ToString();
			}
			else
			{
				parameters = parameters.Replace("{minutes}", minutes.ToString());
			}

			Logger.Verbose("ExternalRecorder[" + this.GetHashCode() + "]: startProgram-Process.Start('" + config.executable + "','" + parameters + "')");

			ProcessStartInfo startInfo = new ProcessStartInfo(config.executable, parameters);
			if (!previewMode) 
			
			{
				startInfo.WindowStyle = ProcessWindowStyle.Hidden;
			}

			process = Process.Start(startInfo);
			process.PriorityClass = ProcessPriorityClass.Idle;

			if (!previewMode)
			{
				for (int i = 0; (i < 50) && !File.Exists(filename); i++)
				{
					System.Threading.Thread.Sleep(100);
				}
			}

			currentStatus = RecordingStatus.RECORDING_IN_PROGRESS;

		}

		#endregion

		#region IRecordingSource Members

		public override void startRecording(IRecordingSourceConfiguration config, int qualityLevel, string filename, ChannelTuning channelTuning, int minutes, bool useStreamBufferEngineIfAvailable)
		{
			Logger.Verbose("ExternalRecorder[" + this.GetHashCode() + "]: enter startRecording(qualityLevel:=" + qualityLevel + ",filename:=" + filename + ",channelTuning:={" + channelTuning.analogChannelNumber.ToString() + "," + channelTuning.digitalTuningRequest + "},minutes:=" + minutes + ",useStreamBufferEngineIfAvailable:=" + useStreamBufferEngineIfAvailable + ")");
			// allocate resources and start recording channelTuning.analogChannelNumber to specified file			
			startProgram(this.config.recordingParameters, filename, channelTuning, minutes, false);
		}

		public override void startNewFile(string filename, ChannelTuning channelTuning)
		{
			Logger.Verbose("ExternalRecorder[" + this.GetHashCode() + "]: enter startNewFile(filename:=" + filename + ", channelTuning:={" + channelTuning.analogChannelNumber.ToString() + "," + channelTuning.digitalTuningRequest + "})");
			// stop graph
			stopRecording();

			// change to channel specified in channelTuning.analogChannelNumber

			// set new file name

			// start graph
			startProgram(config.recordingParameters, filename, channelTuning, 0, false);
		}

		public override RecordingStatus getRecordingStatus()
		{
			return currentStatus;
		}

		public override void stopRecording()
		{
			Logger.Verbose("ExternalRecorder[" + this.GetHashCode() + "]: enter stopRecording()");
			// stop graph and free resources
			if (process != null) 
			{
				if (!process.HasExited) 
				{
					Logger.Verbose("ExternalRecorder[" + this.GetHashCode() + "]: stopRecording-Process.Kill()");
					process.Kill();
				} 
				else 
				{
					Logger.Verbose("ExternalRecorder[" + this.GetHashCode() + "]: stopRecording-Process.HasExited=true");
				}
			}
			currentStatus = RecordingStatus.RECORDING_COMPETED;
		}

		public override string getFinalFileName()
		{
			// you can return a different name here - e.g. if you're recording to a .avi file instead of .mpg 
			return filename;
		}

		public override bool canRecordFMRadio()
		{
			// FM radio not supported
			return false;
		}

		public override void startFMRecording(string filename, int freq)
		{
			// not supported, so do nothing
		}

		public override string getUniqueDeviceIdentifier()
		{
			// this should return a unique string for each physical device. 
			return "ExternalRecorder:" + config.oid.ToString();
		}


		public override PreviewGraph getPreviewGraph(ChannelTuning channelTuning)
		{
			Logger.Verbose("ExternalRecorder[" + this.GetHashCode() + "]: enter getPreviewGraph(channelTuning:={" + channelTuning.analogChannelNumber.ToString() + "," + channelTuning.digitalTuningRequest + "})");
			startProgram(config.previewParameters, string.Empty, channelTuning, 0, true);
			// This should return a PreviewGraph object, with the graphBuilder attribute
			// containing the IFilterGraph2 pointer. The graph should not be run() yet.
			// The other attributes contain the names of a filter, and its pins, which 
			// can be render()'d to preview the graph

			return new PreviewGraph();
		}

		public override void setPreviewToChannel(ChannelTuning channelTuning)
		{
			Logger.Verbose("ExternalRecorder[" + this.GetHashCode() + "]: setPreviewToChannel(channelTuning:={" + channelTuning.analogChannelNumber.ToString() + "," + channelTuning.digitalTuningRequest + "})");
			// This should channel to the channel specified in channelTuning.analogChannelNumber
			stopRecording();
			startProgram(config.previewParameters, string.Empty, channelTuning, 0, true);
		}

		#endregion

		#region IRecordingSourceConfiguration Pass-Through Members

		public void Save(System.Data.OleDb.OleDbConnection connection, int captureSourceOID)
		{
			config.Save(connection, captureSourceOID);
		}

		public void Delete(System.Data.OleDb.OleDbConnection connection, int captureSourceOID)
		{
			config.Delete(connection, captureSourceOID);
		}

		public void Load(int captureSourceOID)
		{
			config.Load(captureSourceOID);
		}

		public string getName()
		{
			return config.getName() + " (IRecordingSourceConfiguration Pass-Through)";
		}

		public string getConfigurationFormClassName()
		{
			return config.getConfigurationFormClassName();
		}

		public IRecordingSource getRecordingSource()
		{
			return config.getRecordingSource();
		}

		#endregion

	}
}
